// mpe.js
//
// MyPhoneExplorer XPCOM Komponente
//

const MPE_ADDON_VERSION = "0.9.18";

// Hilfsfunktionen

// Quelle: http://developer.mozilla.org/en/docs/Code_snippets:File_I/O
//         http://developer.mozilla.org/en/docs/Writing_textual_data

function WriteFile(path,data)
{
	try {
		var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		converter.charset = "UTF-8";
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(path);
		var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
		var chunk = converter.ConvertFromUnicode(data);
		if(chunk !=null) foStream.write(chunk, chunk.length);
		foStream.close();
	}
	catch(ex) {
		Components.utils.reportError('Error writing ' + path + '\n' + (chunk != null ? chunk.length : '-') + ' Bytes: ' + ex);
	}
}

// Quelle: http://developer.mozilla.org/en/docs/Code_snippets:File_I/O
//         http://developer.mozilla.org/en/docs/Reading_textual_data
function ReadFile(path)
{
	var lines = new Array;
	var counter = 0;
	
	try {
		var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		converter.charset = "UTF-8";
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(path);
		var fiStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
		fiStream.init(file, -1, -1, 0);
		
		var lis = fiStream.QueryInterface(Components.interfaces.nsILineInputStream);
		var lineData = {};
		var cont;
		do {
			cont = lis.readLine(lineData);
			lines[counter] = converter.ConvertToUnicode(lineData.value);
			counter++;
		} while (cont);
		fiStream.close();
	}
	catch(ex) {
			Components.utils.reportError('Error reading ' + path + ': ' + ex);
	}
	
	return(lines);
}

//Quelle: http://developer.mozilla.org/en/docs/Accessing_the_Windows_Registry_Using_XPCOM
function RegRead(subkey,valuename)
{
	var out = '';

	try {
		var wrk = Components.classes["@mozilla.org/windows-registry-key;1"].createInstance(Components.interfaces.nsIWindowsRegKey);
		wrk.open(wrk.ROOT_KEY_CURRENT_USER,subkey,wrk.ACCESS_READ);
		out = wrk.readStringValue(valuename);
		wrk.close();
	}
	catch(ex) {
	// 	Components.utils.reportError('Error reading HKCU\\' + subkey + ' : ' + valuename + ' : ' + ex);
	}
	return(out);	
}

// UUID-Generator inspired by http://www.scriptsearch.com/cgi-bin/jump.cgi?ID=3530

function CreateUUID()
{
	function returnBase(num, base){
		var convert = ['0','1','2','3','4','5','6','7','8','9',
		               'A','B','C','D','E','F','G','H','I','J','K','L','M',
		               'N','O','P','Q','R','S','T','U','V','W','X','Y','Z'];

		var output;

    		if(num < base) {
			output = convert[num];
    		} else {
        		var MSD = '' + Math.floor(num / base);
        		var LSD = num - MSD*base;
        		if(MSD >= base) {
				output = returnBase(MSD,base) + convert[LSD];
			} else {
				output = convert[MSD] + convert[LSD];
			}
    		}
    		return output;
	}

	function getIntegerBits(val,start,end)
	{
		var base16 = returnBase(val,16);
		var quadArray = new Array();
		var quadString = '';
		var i;

		for(i = 0;i < base16.length;i++) {
			quadArray.push(base16.substring(i,i+1));	
		}

		for(i = Math.floor(start/4);i <= Math.floor(end/4);i++) {
			if(!quadArray[i] || quadArray[i] == '') {
				quadString += '0';
			} else {
				quadString += quadArray[i];
			}
		}
		return quadString;
	}	

	function timeInMs(d)
	{
		var ms_per_second = 1000; 
		var ms_per_minute = 60000; 
		var ms_per_hour   = 3600000; 
		var ms_per_day    = 86400000; 
		var ms_per_month  = 2073600000; 
		var ms_per_year   = 756864000000;

		return Math.abs((d.getUTCFullYear() * ms_per_year) + (d.getUTCMonth() * ms_per_month) + 
		       (d.getUTCDate() * ms_per_day) + (d.getUTCHours() * ms_per_hour) + 
	               (d.getUTCMinutes() * ms_per_minute) + (d.getUTCSeconds() * ms_per_second) + d.getUTCMilliseconds());
	}

	function randrange(min,max)
	{
		var num = Math.round(Math.random() * max);

		if(num < min) { 
			num = min;
		} else if(num > max) {
			num = max;
		}

		return num;
	}

	var dg = timeInMs(new Date(1582, 10, 15, 0, 0, 0, 0));
	var dc = timeInMs(new Date());
	var t = dc - dg;
	var h = '-';
	var tl = getIntegerBits(t,0,31);
	var tm = getIntegerBits(t,32,47);
	var thv = getIntegerBits(t,48,59) + '1';
	var csar = getIntegerBits(randrange(0,4095),0,7);
	var csl = getIntegerBits(randrange(0,4095),0,7);

	var n = getIntegerBits(randrange(0,8191),0,7) + getIntegerBits(randrange(0,8191),8,15) + 
			getIntegerBits(randrange(0,8191),0,7) + getIntegerBits(randrange(0,8191),8,15) + 
			getIntegerBits(randrange(0,8191),0,15);

	return tl + h + tm + h + thv + h + csar + csl + h + n; 
}
	
function GetPref(prefname,defaultvalue)
{
	var prefvalue = defaultvalue;

	try {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                		getService(Components.interfaces.nsIPrefBranch);

		var preftype = prefs.getPrefType(prefname);

		if(preftype == prefs.PREF_STRING) {
			prefvalue = prefs.getCharPref(prefname);
		}
		else if(preftype == prefs.PREF_BOOL) {
			prefvalue = prefs.getBoolPref(prefname);
		}
		else if(preftype == prefs.PREF_INT) {
			prefvalue = prefs.getIntPref(prefname);
		}
	}
	catch (ex) {
		Components.utils.reportError(ex);
	}

	return prefvalue;
}
		
function EncodeQP(str)
{
	str = escape(str); //aus sonderzeichen wird %00, usw...
	str = str.replace(/%([A-F0-9])/g, "=$1"); // %00 in =00 umwandeln
	str = unescape(str); // Unicode-Zeichen wieder zurckverwandeln (z.B.: %u5498)
	return str;
}
	
function DecodeQP(str)
{
	str = str.replace(/\=/g, "%");  // aus =00 wird %00
	str = unescape(str); // aus %00 wird zeichen
	return str;
}

function AddV(tag, value)
{
	var out = '';
	if (value != null) {
		value = value + '';
		if(value.length) {
			out = ";" + tag + ":" + EncodeQP(value);
		}
	}

	return(out);
}

function GetV(tag, line)
{
	var pos = line.indexOf(tag + ':');

	if(pos > -1) {
		var str = line.slice(pos+tag.length+1);
		pos = str.indexOf(';');

		if(pos == -1) pos = str.indexOf("\r");

		if(pos > -1) {
			str = DecodeQP(str.substring(0,pos));
			if(str == '') str = null;
			return str;
		}
	}

	return null;
}

function SplitV(line)
{
	var tag;
	var value;
	var pos;
	var aarray = new Object();

	var v = line.split(';');

	for(var i = 0;i < v.length;i++) {
		pos = v[i].indexOf(':');
		if(pos <= 0) continue;

		tag = v[i].substring(0,pos);
		value = DecodeQP(v[i].substring(pos+1));
		if(value == '') value = null;

		aarray[tag] = value;
	}

	return aarray;
}

function Value(tag, aarray)
{
	if(tag in aarray)
		return aarray[tag];
	else
		return null;
}

function ValueNN(tag, aarray)
{
	var value = Value(tag,aarray);

	return (value != null) ? value : '';
}

function CardToLine(Card, ABook)
{
	var out = '';

	if(ABook != null) {
		out = 'ABKID:' + EncodeQP(ABook) + ';';
	}

	Card = Card.QueryInterface(Components.interfaces.nsIAbCard);

	out += 'CARID:' + EncodeQP(Card.phoneticLastName) + ';LEDIT:' + Card.lastModifiedDate + AddV('FNAME', Card.firstName)
		+ AddV('LNAME', Card.lastName)  + AddV('DNAME', Card.displayName) + AddV('DMAIL', Card.defaultEmail)
		+ AddV('PMAIL', Card.primaryEmail) + AddV('SMAIL', Card.secondEmail) + AddV('TWORK', Card.workPhone)
		+ AddV('TCELL', Card.cellularNumber) + AddV('THOME', Card.homePhone) + AddV('TEFAX', Card.faxNumber)
		+ AddV('TPAGE', Card.pagerNumber) + AddV('HSTR1', Card.homeAddress) + AddV('HSTR2', Card.homeAddress2)
		+ AddV('HCITY', Card.homeCity) + AddV('HSTAT', Card.homeState) + AddV('HZIPC', Card.homeZipCode)
		+ AddV('HCOUN', Card.homeCountry) + AddV('WSTR1', Card.workAddress) + AddV('WSTR2', Card.workAddress2)
		+ AddV('WCITY', Card.workCity) + AddV('WSTAT', Card.workState) + AddV('WZIPC', Card.workZipCode)
		+ AddV('WCOUN', Card.workCountry) + AddV('DADDR', Card.defaultAddress) + AddV('WURL1', Card.webPage1)
		+ AddV('WURL2', Card.webPage2) + AddV('TITLE', Card.jobTitle) + AddV('COMPA', Card.company)
		+ AddV('BYEAR', Card.birthYear) + AddV('BMONT', Card.birthMonth) + AddV('BIDAY', Card.birthDay)
		+ AddV('NOTES', Card.notes);
	return out;
}

function LineToCard(Card, Line)
{
	var vs = SplitV(Line);

	Card = Card.QueryInterface(Components.interfaces.nsIAbCard);
	
	Card.phoneticLastName = ValueNN("CARID",vs);
	Card.firstName        = ValueNN("FNAME",vs);
	Card.lastName         = ValueNN("LNAME",vs);
	Card.displayName      = ValueNN("DNAME",vs);
	try { Card.defaultEmail = ValueNN("DMAIL",vs); } catch (ex) {}
	Card.primaryEmail     = ValueNN("PMAIL",vs);
	Card.secondEmail      = ValueNN("SMAIL",vs);
	Card.workPhone        = ValueNN("TWORK",vs);
	Card.homePhone        = ValueNN("THOME",vs);
	Card.faxNumber        = ValueNN("TEFAX",vs);
	Card.cellularNumber   = ValueNN("TCELL",vs);
	Card.pagerNumber      = ValueNN("TPAGE",vs);
	Card.homeAddress      = ValueNN("HSTR1",vs);
	Card.homeAddress2     = ValueNN("HSTR2",vs);
	Card.homeState        = ValueNN("HSTAT",vs);
	Card.homeZipCode      = ValueNN("HZIPC",vs);
	Card.homeCity         = ValueNN("HCITY",vs);
	Card.homeCountry      = ValueNN("HCOUN",vs);
	Card.workAddress      = ValueNN("WSTR1",vs);
	Card.workAddress2     = ValueNN("WSTR2",vs);
	Card.workState        = ValueNN("WSTAT",vs);
	Card.workZipCode      = ValueNN("WZIPC",vs);
	Card.workCity         = ValueNN("WCITY",vs);
	Card.workCountry      = ValueNN("WCOUN",vs);
	Card.defaultAddress   = ValueNN("DADDR",vs);
	Card.webPage1         = ValueNN("WURL1",vs);
	Card.webPage2         = ValueNN("WURL2",vs);
	Card.jobTitle         = ValueNN("TITLE",vs);
	Card.company          = ValueNN("COMPA",vs);
	Card.birthYear        = ValueNN("BYEAR",vs);
	Card.birthMonth       = ValueNN("BMONT",vs);
	Card.birthDay         = ValueNN("BIDAY",vs);
	Card.notes            = ValueNN("NOTES",vs);
}

function CardsSimpleEnum() {};

CardsSimpleEnum.prototype = {
	buffer : null,
	atend : false,
	enumerator : null,

	hasMoreElements : function () {
		if(this.buffer != null) return true;

		if(this.atend) return false;

		try {
			this.buffer = this.enumerator.currentItem();

			try {
				this.enumerator.next();
			}
			catch (ex) {
				this.atend = true;
			}

			return true;
		}
		catch (ex) {}

		return false;
	},

	getNext : function () {
		if(this.buffer != null) {
			var tmp = this.buffer;
			this.buffer = null;
			return tmp;
		}
		else {
			throw("hasMoreElements() was not called");
		}
	}
};

function GetCards(ab)
{
	var cards_enum = ab.childCards;

	try {
		cards_enum.hasMoreElements();
	}
	catch(ex) {
		try {
			cards_enum.first();
		}
		catch(ex) {}

		var cse = new CardsSimpleEnum();

		cse.enumerator = cards_enum;

		return cse;
	}

	return cards_enum;
}

function GetNextValidCard(cards_enum)
{
	var card;

	while(cards_enum.hasMoreElements()) {
		card = cards_enum.getNext();
		card = card.QueryInterface(Components.interfaces.nsIAbCard);
		if (!card.isMailList) {
			return card;
		}
	}
	return null;
}

function GetABURI(dir)

{
	try {
		return dir.directoryProperties.URI; // thunderbird <= 2
	}
	catch(ex) {
		return dir.URI;	// thunderbird 3
	}
}

function SaveCard(ab,card)
{
	try {
		card.editCardToDatabase(GetABURI(ab));
	}
	catch (ex) {}
}

function GetAddressBook(abook)
{
	var ab;
	var abName;

	var Rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
	var RootDir = Rdf.GetResource("moz-abdirectory://").QueryInterface(Components.interfaces.nsIAbDirectory);
	var SubDirs = RootDir.childNodes.QueryInterface(Components.interfaces.nsISimpleEnumerator);

	while (SubDirs.hasMoreElements()) {
		ab = SubDirs.getNext().QueryInterface(Components.interfaces.nsIAbDirectory);
		if (!ab.isMailList) {
			abName = GetABURI(ab);
			if (abook.length > 0) {
				if (abName == abook) return ab;
			}
			else {
				if (abName.indexOf("//abook.mab") >= 0) {
					return ab;
				}
			}
		}
	}
	
	throw("Addressbook " + abook + " not found !");
}

function ScanCards(ab)
{
	var cards = new Object();

	var cards_enum = GetCards(ab);
	var curCard;

	while(curCard = GetNextValidCard(cards_enum)) {
		if(curCard.phoneticLastName == null ||
		   !curCard.phoneticLastName.match(/\w{8}-\w{4}-\w{4}-\w{4}-\w{12}/)) {
			curCard.phoneticLastName = CreateUUID();
			SaveCard(ab,curCard);
		}

		cards[curCard.phoneticLastName] = curCard;
	}

	return cards;
}
	
function FindCard(ab,cards,carid)
{
	if(carid in cards)
		return cards[carid];
	else
		return null;
}

function DeleteCard(ab,cards,carid)
{
	if(carid in cards && cards[carid] != null) {
		var delArray = Components.classes["@mozilla.org/supports-array;1"].
				createInstance(Components.interfaces.nsISupportsArray);

		delArray.AppendElement(cards[carid]);
		cards[carid] = null;

		ab.deleteCards(delArray);
	}
}

function isEvent(aObject) { return aObject instanceof Components.interfaces.calIEvent; }

function isToDo(aObject) { return aObject instanceof Components.interfaces.calITodo; }

function Date2String(dt)
{
	try {
		var dt2 = dt.clone();
		var ltz =  GetPref("calendar.timezone.local","UTC");

		if(dt.timezone == "floating" || (dt.isDate && dt.timezone != ltz)) {
			dt2.resetTo(dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second,ltz);
			dt2.isDate = dt.isDate;
		} 
			
		var str = dt2.nativeTime+"";
		return (dt2.isDate ? 'D' : '') + str.slice(0,-6);
	}
	catch (ex) {
		return '';
	}
}

function String2Date(str)
{
	if(str == null) return null;

	var dt = null;

	try {
		dt = Components.classes["@mozilla.org/calendar/datetime;1"].
				createInstance(Components.interfaces.calIDateTime);

		var isdate = (str.slice(0,1) == 'D');
		var nativetime = (isdate ? str.slice(1) : str) + '000000';		

		dt.nativeTime = nativetime;	// setzt UTC

		dt = dt.getInTimezone(GetPref("calendar.timezone.local","UTC"));	// Kalender-Zeitzone

		dt.isDate = isdate;		// erst jetzt darf man isDate setzen

		if(dt.nativeTime != nativetime) {	// vermutlich falsche Zeitzone im Kalender
			dt.nativeTime = nativetime;	// dann lieber den Termin richtig statt ganztaegig und falsch
		}
	}
	catch (ex) {
		Components.utils.reportError(ex);
	}

	return dt;	
}

function RItem2String(ri)
{
	return ri.icalProperty.icalString;
}

function ItemRecurrence(Item)
{
	try {
		var riarray = Item.recurrenceInfo.getRecurrenceItems({});
		var str = '';

		for(var i = 0;i < riarray.length;i++) {
			str += RItem2String(riarray[i]) + "\r\n";
		}

		return AddV('RINFO',str);
	}
	catch (ex) {
		//Components.utils.reportError(ex);
		return '';
	}
}

function ParseRecurrence(item,rinfostr)
{
	try {
		var ICSService = Components.classes["@mozilla.org/calendar/ics-service;1"].
					getService(Components.interfaces.calIICSService);

		var rinfo = Components.classes["@mozilla.org/calendar/recurrence-info;1"].
					createInstance(Components.interfaces.calIRecurrenceInfo);

		var ritem;

		rinfo.item = item;

		var icalcomp = ICSService.parseICS("BEGIN:VEVENT\r\n" + rinfostr + "\r\nEND:VEVENT\r\n");

		for(var recprop = icalcomp.getFirstProperty("ANY");
		    recprop;
		    recprop = icalcomp.getNextProperty("ANY")) {

			if(recprop.propertyName == "RRULE" || recprop.propertyName == "EXRULE") {
				ritem = Components.classes["@mozilla.org/calendar/recurrence-rule;1"].
						createInstance(Components.interfaces.calIRecurrenceRule);
			}
			else if(recprop.propertyName == "RDATE" || recprop.propertyName == "EXDATE") {
				ritem = Components.classes["@mozilla.org/calendar/recurrence-date;1"].
						createInstance(Components.interfaces.calIRecurrenceDate);
			}
			else continue;

			ritem.icalProperty = recprop;
			rinfo.appendRecurrenceItem(ritem);
		}

		return rinfo;
	}
	catch (ex) {
		Components.utils.reportError(ex);
	}

	return null;
}

function ItemAlarm(Item)
{
	try {
		return ';ALTYP:' + (Item.alarmRelated == Item.ALARM_RELATED_START ? 'START' : 'END') +
		       ';ALOFF:' + Item.alarmOffset.inSeconds;
	}
	catch (ex) {
		return '';
	}
}

function ItemToLine(Item, ReadOnly)
{
	var line = '';
	var calid = Item.calendar.uri.asciiSpec;

	if(isEvent(Item)) {
		line = 'ITYPE:EVENT' + AddV('CALID',calid)
		  + AddV('ITMID',Item.id) + AddV('TITLE',Item.title) 
		  + AddV('DSCPN',((calid.slice(0,19) == 'moz-abmdbdirectory:') ? Item.startDate.year : Item.getProperty('DESCRIPTION')))
		  + AddV('LOCTN',Item.getProperty('LOCATION'))
	          + AddV('SDATE',Date2String(Item.startDate)) + AddV('EDATE',Date2String(Item.endDate))
	          + AddV('CTIME',Date2String(Item.creationDate)) + AddV('MTIME',Date2String(Item.lastModifiedTime))
	          + AddV('PRIOR',Item.priority) + AddV('PRVCY',Item.privacy)
	          + AddV('RONLY',ReadOnly ? '1' : null)
	          + AddV('STATS',Item.status) + ItemAlarm(Item) + ItemRecurrence(Item) + '\r\n';
	}
	else if(isToDo(Item)) {
		line = 'ITYPE:TODO' + AddV('CALID',calid)
		  + AddV('ITMID',Item.id) + AddV('TITLE',Item.title)
		  + AddV('DSCPN',Item.getProperty('DESCRIPTION')) + AddV('LOCTN',Item.getProperty('LOCATION'))
	          + AddV('SDATE',Date2String(Item.entryDate)) + AddV('EDATE',Date2String(Item.dueDate))
	          + AddV('CMPLT',(Item.isCompleted ? 'YES' : 'NO'))
	          + AddV('CTIME',Date2String(Item.creationDate)) + AddV('MTIME',Date2String(Item.lastModifiedTime))
	          + AddV('PRIOR',Item.priority) + AddV('PRVCY',Item.privacy)
	          + AddV('RONLY',ReadOnly ? '1' : null)
	          + AddV('STATS',Item.status) + ItemAlarm(Item) + ItemRecurrence(Item) + '\r\n';
	}
	else {
		Components.utils.reportError("Unknown item type: " + Item.id);
	}

	return line;
}

function LineToItem(Item,Line)
{
	var vs = SplitV(Line);

	var itype  = Value("ITYPE",vs);

	Item.id	   = Value("ITMID",vs);
	Item.title = Value("TITLE",vs);

	if(Value("DSCPN",vs) != null && Value("CALID",vs).slice(0,19) != 'moz-abmdbdirectory:')
		Item.setProperty('description',Value("DSCPN",vs));
	else
		Item.deleteProperty('description');

	if(Value("LOCTN",vs) != null)
		Item.setProperty('location',Value("LOCTN",vs));
	else
		Item.deleteProperty('location');

	if(itype == "TODO")
		Item.isCompleted = (Value("CMPLT",vs) == "YES");

	Item.priority = Value("PRIOR",vs);
	Item.privacy  = Value("PRVCY",vs);
	Item.status   = Value("STATS",vs);

	if(itype == "EVENT")
		Item.startDate = String2Date(Value("SDATE",vs));
	else 
		Item.entryDate = String2Date(Value("SDATE",vs));

	if(itype == "EVENT")
		Item.endDate = String2Date(Value("EDATE",vs));
	else 
		Item.dueDate = String2Date(Value("EDATE",vs));

	if(Value("ALTYP",vs) != null)
		Item.alarmRelated = (Value("ALTYP",vs) == "START" ? Item.ALARM_RELATED_START : Item.ALARM_RELATED_END);
	else
		Item.alarmRelated = null;

	if(Value("ALOFF",vs) != null) {
		Item.alarmOffset = Components.classes["@mozilla.org/calendar/duration;1"].
					createInstance(Components.interfaces.calIDuration);
		Item.alarmOffset.inSeconds = Value("ALOFF",vs);
	}
	else
		Item.alarmOffset = null;

	if(Value("RINFO",vs) != null) {
		Item.recurrenceInfo = ParseRecurrence(Item,Value("RINFO",vs));
	}
	else
		Item.recurrenceInfo = null;
}

function GetCalendar(calendar)
{
	var calendarManager = Components.classes["@mozilla.org/calendar/manager;1"]
               	                     .getService(Components.interfaces.calICalendarManager);
	var cals = calendarManager.getCalendars({});

	var c;

	for(c = 0;c < cals.length;c++) { // Zuerst gegen die URI matchen
		if(cals[c].uri.asciiSpec == calendar) {
			return cals[c];
		}
	}

	for(c = 0;c < cals.length;c++) { // dann gegen den Namen
		if(cals[c].name == calendar) {
			return cals[c];
		}
	}
	
	throw("Calender " + calendar + " not found!");
}

// XPCOM

const mpeIMyPhoneExplorer	= Components.interfaces.mpeIMyPhoneExplorer;
const nsISupports		= Components.interfaces.nsISupports;
const CLASS_ID			= Components.ID("{cd51fa3f-2663-49d0-91c9-fdf81fb87fc2}");
const CLASS_NAME		= "MyPhoneExplorer XPCOM Component";
const CONTRACT_ID		= "@fjsoft.at/MyPhoneExplorer;1";

function MyPhoneExplorer() {};

MyPhoneExplorer.prototype = {

QueryInterface: function(aIID)
{
	if (!aIID.equals(mpeIMyPhoneExplorer) && !aIID.equals(nsISupports))
		throw Components.results.NS_ERROR_NO_INTERFACE;
	return this;
},

//
// Allgemeines
//

get VERSION() { return MPE_ADDON_VERSION; },

ListInfo: function(filename)
{
	WriteFile(filename,"Version:     "+this.VERSION+"\r\n"+
	                   "Addressbook: "+(this.HasABook ? "YES" : "NO")+"\r\n"+
	                   "Calendar:    "+(this.HasCalendar ? "YES" : "NO")+"\r\n");
},

CWOS_State : false,
CWOS_Time : 0,

get CloseWindowOnStartup()

{
	// check for fixed Sunbird version

	try {
		var catman = Components.classes["@mozilla.org/categorymanager;1"]
                	           .getService(Components.interfaces.nsICategoryManager);

		if(catman.getCategoryEntry("command-line-handler","m-calendar")
		   != "@mozilla.org/commandlinehandler/general-startup;1?type=calendar") {
			this.CWOS_State = false;
		}
	}
	catch(ex) {
		this.CWOS_State = false;
	}

	if(this.CWOS_State) {
		var now = Date.now();

		if(now - this.CWOS_Time > 5000) {
			this.CWOS_State = false;
		}
	}

	return this.CWOS_State;
},

set CloseWindowOnStartup(state)

{
	this.CWOS_State = state;
	this.CWOS_Time = Date.now();
},

// Quelle: http://developer.mozilla.org/en/docs/Code_snippets:Running_applications
Run: function(args)
{
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
	var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
	
	var path = RegRead("Software\\MyPhoneExplorer", "PathToExe");
	if (path.length != 0) {
		try
		{
			file.initWithPath(path);
			process.init(file);
			process.run(false, [args], 1);
			return;
		}
		catch(ex)
		{
			Components.utils.reportError('Could not run ' + path);
			return;
		}
	}
	Components.utils.reportError("Could not find MyPhoneExplorer.exe");
},

SendMessage: function(num)
{
	this.Run("action=sendmessage number="+num);
},

DialNumber: function(num)
{
	this.Run("action=dial number="+num);
},

Notify: function(action)
{
	this.Run("mozilla-notify="+action);
},

SyncAll: function()
{
	this.Run("action=sync syncitem=multi");
},

GetFilePath: function(filename)
{
	var filepath = RegRead("Software\\MyPhoneExplorer\\Mozilla", filename);

	if(filepath != '') return filepath;

	filepath = RegRead("Software\\MyPhoneExplorer", "Database");
	if(filepath == '') {
		filepath = RegRead("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", "AppData");
	}

	if(filepath != '') {
		filepath += "\\MyPhoneExplorer\\" + filename;
	}

	return filepath;
},

GetRegEntry: function(keyvalue)
{
	var key = "Software\\MyPhoneExplorer";
	var value;

	var n = keyvalue.lastIndexOf("\\");

	if(n > 0) key += "\\" + keyvalue.slice(0,n-1);
	value = keyvalue.slice(n+1);

	return RegRead(key,value);
},

//
// Adressbuchfunktionen
//

get HasABook()
{
	try {
		var Rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
		var RootDir = Rdf.GetResource("moz-abdirectory://").QueryInterface(Components.interfaces.nsIAbDirectory);

		if(RootDir != null) return true;
	}
	catch (ex) {}

	return false;
},

get ABookLock() { return (RegRead("Software\\MyPhoneExplorer\\Mozilla", "ABLock") == 1); },

ListABooks: function(filename)
{
	var folderlist = '';

	try {
		var Rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
		var RootDir = Rdf.GetResource("moz-abdirectory://").QueryInterface(Components.interfaces.nsIAbDirectory);
		var SubDirs = RootDir.childNodes.QueryInterface(Components.interfaces.nsISimpleEnumerator);
  
		while (SubDirs.hasMoreElements()) {
			var dir = SubDirs.getNext().QueryInterface(Components.interfaces.nsIAbDirectory);

			if (!dir.isMailList) {
				folderlist += dir.dirName + "\t" + GetABURI(dir) + "\r\n";
			}
		}
	}
	catch (ex) {}

	WriteFile(filename,folderlist);
},

get ChosenABook() { return RegRead("Software\\MyPhoneExplorer\\Mozilla", "AddressBook"); },

WriteCards: function(abook,filename)
{
	var abooks = abook.split('|');
	var cards = 'AddOn-Version='+MPE_ADDON_VERSION+'\r\n';

	var ab;

	for(var i = 0;i < abooks.length;i++) {
		ab = GetAddressBook(abooks[i]);

		var cards_enum = GetCards(ab);
		var nextCard;

		while(nextCard = GetNextValidCard(cards_enum)) {
			if(nextCard.phoneticLastName == null ||
			  !nextCard.phoneticLastName.match(/\w{8}-\w{4}-\w{4}-\w{4}-\w{12}/)) {
				nextCard.phoneticLastName = CreateUUID();
				SaveCard(ab,nextCard);
			}

			cards += CardToLine(nextCard,abooks[i]) + '\r\n';
		}
	}

	WriteFile(filename,cards);
},

ReadCards: function(abook,filename)
{
	var abooks = abook.split('|');

	var i;
	var ab_lookup = new Object();
	var ab = new Array();
	var ab_cards = new Array();

	for(i = 0;i < abooks.length;i++) {
		ab[i] = GetAddressBook(abooks[i]);
		ab_lookup[abooks[i]] = i;
		ab_cards[i] = ScanCards(ab[i]);
	}

	var lines = ReadFile(filename);
	var op;
	var card;
	var carid;
	var cur_abook;
	var cur_ab_i;
	
	for (i = 1;lines.length > i;i++) { // Erste Zeile nicht bentigt, weil Versionsinfo
		op = lines[i].substring(0,lines[i].indexOf(';'));

		cur_abook = GetV('ABKID',lines[i]);
		if(cur_abook in ab_lookup)
			cur_ab_i = ab_lookup[cur_abook];
		else
			cur_ab_i = 0;

		carid = GetV('CARID',lines[i]);
		if(carid != null && carid != '')
			card = FindCard(ab[cur_ab_i],ab_cards[cur_ab_i],carid);
		else
			card = null;

		if(op == "NEW" || op == "EDIT") {
			if(card != null) {
				op = "EDIT";
			}
			else {
				card = Components.classes["@mozilla.org/addressbook/cardproperty;1"].
						createInstance(Components.interfaces.nsIAbCard);
				op = "NEW";
			}

			LineToCard(card,lines[i]);

			if(op == "NEW")
				ab[cur_ab_i].addCard(card);
			else
				SaveCard(ab[cur_ab_i],card);
		}
		else if(op == "DELETE") {
			if(card != null) {
				DeleteCard(ab[cur_ab_i],ab_cards[cur_ab_i],carid);
			}
		}
		else if(op.slice(0,9) == "IDCHANGE=") {
			if(card != null) {
				card.phoneticLastName = op.slice(9);
				SaveCard(ab[cur_ab_i],card);
			}
		}
	}
},

RAB: "",

get RequestedABook()
{
	return this.RAB;
},

set RequestedABook(ab)
{
	this.RAB = ab;
},

RCI: "",

get RequestedCardId()
{
	return this.RCI;
},

set RequestedCardId(id)
{
	this.RCI = id;
},

GetCard: function(abook,id)
{
	if(abook == null || abook.value == null || abook.value == '' ||
	   id == null || id == '') { return null; }

	var abooks = abook.value.split('|');

	for(i = 0;i < abooks.length;i++) {
		var ab = GetAddressBook(abooks[i]);

		var cards_enum = GetCards(ab);
		var curCard;
	
		while(curCard = GetNextValidCard(cards_enum)) {
			if(id.match(/\w{8}-\w{4}-\w{4}-\w{4}-\w{12}/)) {
				if(curCard.phoneticLastName == id) {
					abook.value = abooks[i];
					return curCard;
				}
			} else if(curCard.displayName == id) {
				abook.value = abooks[i];
				return curCard;
			} else if(curCard.firstName + ' ' + curCard.lastName == id) {
				abook.value = abooks[i];
				return curCard;
			}
		}
	}

	return null;
},

SyncABook: function()
{
	this.Run("action=sync syncitem=phonebook");
},

//
// Kalenderfunktionen
//

get HasCalendar()
{
	try {
		var calendarManager = Components.classes["@mozilla.org/calendar/manager;1"]
        	       	                     .getService(Components.interfaces.calICalendarManager);

		if(calendarManager != null) return true;
	}
	catch (ex) {}

	return false;
},

ListCalendars: function(filename)
{
	var callist = '';
	var calcolor; 

	try {
		var calendarManager = Components.classes["@mozilla.org/calendar/manager;1"]
               		                     .getService(Components.interfaces.calICalendarManager);
		var cals = calendarManager.getCalendars({});

		for(var c = 0;c < cals.length;c++) {
			try {
				calcolor = calendarManager.getCalendarPref(cals[c],'color');
			}
			catch(gcp) {
				try {
					calcolor = calendarManager.getCalendarPref_(cals[c],'color');
				}
				catch(gcp2) {
					calcolor = "#ffffff";
				}
			}


			if(calcolor == null || calcolor == '') {
				calcolor = "#a8c2e1";
			}

			callist += cals[c].name + "\t" + cals[c].uri.asciiSpec + "\t"
			           + (cals[c].readOnly ? "RO" : "RW") + "\t" + calcolor.toUpperCase() + "\r\n";
		}
	}
	catch (ex) {
		Components.utils.reportError(ex);
	}

	WriteFile(filename,callist);
},

get ChosenCalendar() { return RegRead("Software\\MyPhoneExplorer\\Mozilla", "Calendar"); },

ItemCache: null,

CalendarCache: null,

CROCache: null,

ItemCacheTimeStamp: 0,

FillItemCache: function(calendar,aac)

{
	var calendars = calendar.split('|');

	var cstat = new Array();

	function CacheFiller() {};

	CacheFiller.prototype = {
		complete_already_called : false,
		mpe: null,
		calIndex: null,
		calCount: null,
		items: null,
		
		onOperationComplete: function(aCalendar, aStatus, aOperationType, aId, aDetail) 
		{
			if(this.complete_already_called) return;
			else this.complete_already_called = true;

			this.mpe.ItemCache[this.calIndex] = this.items;
			cstat[this.calIndex] = true;

			for(var i = 0;i < this.calCount;i++) { // Der Letzte schreibt die Ergebnisse
				if(!cstat[i]) return;
			}

			this.mpe.ItemCacheTimeStamp = Date.now();

			aac.AsyncActionComplete(true);
		},

        	onGetResult: function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems)
        	{
			for(var i = 0;i < aItems.length;i++) {
				this.items.push(aItems[i]);
			}
        	}
	};

	var cf;
	var i;
	var cal;

	for(i = 0;i < calendars.length;i++) {
		cstat[i] = false;
	}

	this.ItemCache = new Array();
	this.CalendarCache = calendar;
	this.CROCache = new Array();
	
	for(i = 0;i < calendars.length;i++) {
		cal = GetCalendar(calendars[i]);
		this.CROCache[i] = cal.readOnly;

		cf = new CacheFiller();
		cf.calIndex = i;
		cf.calCount = calendars.length;
		cf.mpe = this;
		cf.items = new Array();

		cal.getItems(Components.interfaces.calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null, cf);
	}
},

WriteItems: function(calendar,filename,aac)

{
	function ItemWriter() {};
	
	ItemWriter.prototype = {
		mpe: null,

		AsyncActionComplete: function(success)
		{
			var itemdata = 'AddOn-Version='+MPE_ADDON_VERSION+'\r\n';

			for(var i = 0;i < this.mpe.ItemCache.length;i++) {
				for(var j = 0;j < this.mpe.ItemCache[i].length;j++) {
					itemdata += ItemToLine(this.mpe.ItemCache[i][j],this.mpe.CROCache[i]);
				}
			}

			WriteFile(filename,itemdata);

			aac.AsyncActionComplete(true);
		}
	};

	var iw = new ItemWriter();

	iw.mpe = this;

	this.FillItemCache(calendar,iw);
},

ReadItems: function(calendar,filename,aac)

{
	function ItemReader() {};

	ItemReader.prototype = {
		mpe: null,

		AsyncActionComplete: function(success)
		{
			var lines = ReadFile(filename);
			var op;
			var calid;
			var itmid;
			var itype;
			var calIndex;
			var itmIndex;
			var i;
			var j;

			var calendars = this.mpe.CalendarCache.split('|');
			var cals = new Array();
			var item;

			for(i = 0;i < calendars.length;i++) {
				cals[i] = GetCalendar(calendars[i]);
			}
			
			for(i = 1;i < lines.length;i++) { // Erste Zeile nicht benoetigt, weil Versionsinfo
				try {
					calid = GetV('CALID',lines[i]);
					itmid = GetV('ITMID',lines[i]);
					itype = GetV('ITYPE',lines[i]);

					op = lines[i].substring(0,lines[i].indexOf(';'));

					if(op != 'NEW' && op != 'DELETE' && op != 'EDIT') {
						continue;
					}

					if(itmid == null && op != "NEW") {
						throw "Missing item ID in line "+i;
					}

					calIndex = 0;

					if(calid != null) {
						for(j = 0;j < calendars.length;j++) {
							if(calid == calendars[j]) {
								calIndex = j;
								break;
							}
						}
					}

					itmIndex = null;

					for(j = 0;j < this.mpe.ItemCache[calIndex].length;j++) {
						if(this.mpe.ItemCache[calIndex][j] == null) continue;

						if(itmid == this.mpe.ItemCache[calIndex][j].id) {
							itmIndex = j;
							break;
						}
					}

					if(op == "DELETE") {
						if(itmIndex == null) throw "Missing or wrong item ID in line "+i;

						cals[calIndex].deleteItem(this.mpe.ItemCache[calIndex][itmIndex],null);
						this.mpe.ItemCache[calIndex][itmIndex] = null;
					}
					else if(op == "NEW" || op == "EDIT") {
						if(itmIndex != null) {
							item = this.mpe.ItemCache[calIndex][itmIndex].clone();
							if(item == null) throw "Clone of old item failed";
						}
						else {
							if(itype == 'EVENT') {
								item = Components.classes["@mozilla.org/calendar/event;1"].
									createInstance(Components.interfaces.calIEvent);
							}
							else if(itype == 'TODO') {
								item = Components.classes["@mozilla.org/calendar/todo;1"].
									createInstance(Components.interfaces.calITodo);
							}
							else throw "Missing or unknown item type";

							if(item == null) throw "Creation of new item failed";
						}

						LineToItem(item,lines[i]);

						if(itmIndex != null) {
							cals[calIndex].modifyItem(item,this.mpe.ItemCache[calIndex][itmIndex],null);
						}
						else {
							cals[calIndex].addItem(item,null);
						}
					}
				}
				catch(ex) {
					Components.utils.reportError(ex);
				}
			}

			this.mpe.ItemCache = null;
			this.mpe.CalendarCache = null;
			this.mpe.CROCache = null;
			this.mpe.ItemCacheTimeStamp = 0;

			aac.AsyncActionComplete(true);
		}
	};

	var ir = new ItemReader();

	ir.mpe = this;

	var now = Date.now();

	if(this.ItemCache == null || this.CalendarCache != calendar || now - this.ItemCacheTimeStamp > 600000) {
		this.FillItemCache(calendar,ir);
	}
	else {
		ir.AsyncActionComplete(true);
	} 
},

SyncCalendar: function()
{
	this.Run("action=sync syncitem=organizer");
}

};

var MyPhoneExplorerFactory = {
	createInstance: function (aOuter, aIID)
	{
		if (aOuter != null)
			throw Components.results.NS_ERROR_NO_AGGREGATION;
		return (new MyPhoneExplorer()).QueryInterface(aIID);
	}
};

var MyPhoneExplorerModule = {
	_firstTime : true,

	registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
	{
		if (!this._firstTime) throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
		this._firstTime = false;

		aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
	},

	unregisterSelf: function(aCompMgr, aLocation, aType)
	{
		aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
	},
  
	getClassObject: function(aCompMgr, aCID, aIID)
	{
		if (!aIID.equals(Components.interfaces.nsIFactory))
			throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

		if (aCID.equals(CLASS_ID))
			return MyPhoneExplorerFactory;

		throw Components.results.NS_ERROR_NO_INTERFACE;
	},

	canUnload: function(aCompMgr) { return true; }
};

function NSGetModule(aCompMgr, aFileSpec) { return MyPhoneExplorerModule; }
