// mpecli.js
//
// command line arguments for the MPE extension
//

const MPE_CLI_CONTRACTID  = "@mozilla.org/commandlinehandler/general-startup;1?type=mpe";
const MPE_CLI_CLSID       = Components.ID("{ce17443a-943a-401c-8955-ed0c173c729d}");
const MPE_CLI_TITLE       = "MPE CLI Service";
const MPE_CLI_TITLE1      = "MyPhoneExplorer CLI Handler";
const MPE_CLI_HELP        = "Execute MyPhoneExplorer Commands.";
const MPE_CLI_HELP1       = "  -mpe                Execute MyPhoneExplorer Commands.\n";

function mpeCliHandler() {}

mpeCliHandler.prototype =
{

	/* Components.interfaces.nsICmdLineHandler */
	commandLineArgument : "-mpe",
	prefNameForStartup  : "general.startup.mpe",
	helpText            : MPE_CLI_HELP,
	handlesArgs         : true,
	defaultArgs         : "",
	openWindowWithArgs  : false,

	/* Components.interfaces.nsICommandLineHandler */
	handle : function handler_handle(cmdLine) 
	{
		var args = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);

		try {
			var command = cmdLine.handleFlagWithParam("mpe", false);
			var mpe = Components.classes["@fjsoft.at/MyPhoneExplorer;1"].getService(Components.interfaces.mpeIMyPhoneExplorer);
			var com,filename,chosen;
			var notify = true;

			if(command == null) {
				mpe.CloseWindowOnStartup = false;
				return;
			}

			var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].
					getService(Components.interfaces.nsIWindowWatcher);

			var window_opened = false;

			var i = command.indexOf('=');
			if(i >= 0) {
				com = command.substring(i+1).split(';');
				command = command.substring(0,i);
				filename = com[0];
				chosen = com[1];
				notify = (com[2] != 'quiet'); 
			}

			if(command == 'version' || command == 'info' || command == 'list-info') {
				mpe.ListInfo(filename ? filename : mpe.GetFilePath("MozillaInfo"));
				if(notify) mpe.Notify(command);
			} 
			else if(command == 'list-abooks' || command == 'list-addressbooks') {
				mpe.ListABooks(filename ? filename : mpe.GetFilePath("MozillaAddressBooks"));
				if(notify) mpe.Notify(command);
			}
			else if(command == 'export-cards') {
				mpe.WriteCards(chosen ? chosen : mpe.ChosenABook,filename ? filename : mpe.GetFilePath("MozillaCards"));
				if(notify) mpe.Notify(command);
			}
			else if(command == 'import-cards') {
				mpe.ReadCards(chosen ? chosen : mpe.ChosenABook,filename ? filename : mpe.GetFilePath("MozillaCards"));
				if(notify) mpe.Notify(command);
			}
			else if((command == 'showcontact' || command == 'show-card') && mpe.HasABook) {
				if(filename != null && filename != '') { // filename contains card id
					mpe.RequestedCardId = filename;
					mpe.RequestedABook = chosen ? chosen : mpe.ChosenABook;

					ww.openWindow(null, "chrome://myphoneexplorer/content/mpeShowCard.xul",
                                               	      null, 'chrome,centerscreen', null );

					window_opened = true;
				} 
			}
			else if(command == 'list-cals' || command == 'list-calendars') {
				mpe.ListCalendars(filename ? filename : mpe.GetFilePath("MozillaCalendars"));
				if(notify) mpe.Notify(command);
			}
			else if(command == 'export-items') {
				export_items_aac = {
					AsyncActionComplete: function(success) {
						if(notify && success) mpe.Notify('export-items');
					}
				};

				mpe.WriteItems(chosen ? chosen : mpe.ChosenCalendar,
				               filename ? filename : mpe.GetFilePath("MozillaItems"),
				               export_items_aac);
			}
			else if(command == 'import-items') {
				import_items_aac = {
					AsyncActionComplete: function(success) {
						if(notify && success) mpe.Notify('import-items');
					}
				};

				mpe.ReadItems(chosen ? chosen : mpe.ChosenCalendar,
				              filename ? filename : mpe.GetFilePath("MozillaItems"),
				              import_items_aac);
			}
			else {
				Components.utils.reportError("Unknown suboption -mpe " + command);
				return;
			}

			mpe.CloseWindowOnStartup = true;
	  		cmdLine.preventDefault = true;

			if(!window_opened) {
				ww.openWindow(null, "chrome://myphoneexplorer/content/mpeDummyWindow.xul",
			        	      null, null, null);
			}
    		}
		catch (ex) {
			Components.utils.reportError("Exception caught during -mpe "+command+": "+ex);
    		}
  	},

  	helpInfo : MPE_CLI_HELP,
  
	/* Components.interfaces.nsISupports */
  	QueryInterface : function handler_QI(aIID) 
  	{
    		if (aIID.equals(Components.interfaces.nsISupports) ||
        	    (Components.interfaces.nsICmdLineHandler && aIID.equals(Components.interfaces.nsICmdLineHandler)) || 
        	    (Components.interfaces.nsICommandLineHandler && aIID.equals(Components.interfaces.nsICommandLineHandler))) {
      			return this;
    		}

    		throw Components.results.NS_ERROR_NO_INTERFACE;
  	}
};


var mpeCliFactory =
{
	createInstance : function(outer, iid)
	{
    		if (outer != null) {
      			throw Components.results.NS_ERROR_NO_AGGREGATION;
    		}

    		return new mpeCliHandler().QueryInterface(iid);
  	}
};


var mpeCliModule =
{
	registerSelf : function(compMgr, fileSpec, location, type)
  	{
    		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    		compMgr.registerFactoryLocation(MPE_CLI_CLSID, MPE_CLI_TITLE, MPE_CLI_CONTRACTID, fileSpec, location, type);

    		var catman = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
    		catman.addCategoryEntry("command-line-argument-handlers", MPE_CLI_TITLE1, MPE_CLI_CONTRACTID, true, true);
    		catman.addCategoryEntry("command-line-handler", "m-mpe", MPE_CLI_CONTRACTID, true, true);
  	},

  	unregisterSelf : function(compMgr, fileSpec, location)
  	{
    		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    		compMgr.unregisterFactoryLocation(MPE_CLI_CLSID, fileSpec);
    		catman = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
    		catman.deleteCategoryEntry("command-line-argument-handlers", MPE_CLI_TITLE1, true);
    		catman.deleteCategoryEntry("command-line-handler", "m-mpe", true);
  	},

  	getClassObject : function(compMgr, cid, iid)
  	{
    		if (cid.equals(MPE_CLI_CLSID)) {
      			return mpeCliFactory;
    		}

    		if (!iid.equals(Components.interfaces.nsIFactory)) {
      			throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    		}

    		throw Components.results.NS_ERROR_NO_INTERFACE;
  	},

  	canUnload : function(compMgr)
  	{
    		return true;
  	}
};


function NSGetModule(compMgr, fileSpec) {
  	return mpeCliModule;
}
