// mpeproto.hs
//
// protocol handler for XPI Update
//

const MPE_PROTO_CONTRACTID = "@mozilla.org/network/protocol;1?name=mpe";
const MPE_PROTO_CLSID      = Components.ID("{98d4e46a-3969-49af-b732-555ea6d91180}");
const MPE_PROTO_TITLE      = "MPE XPI Update Protocol";

function mpeProtocolHandler() {}

mpeProtocolHandler.prototype = {
	scheme : "mpe",

	defaultPort : -1,

	protocolFlags : Components.interfaces.nsIProtocolHandler.URI_NORELATIVE |
	                Components.interfaces.nsIProtocolHandler.URI_NOAUTH,

	allowPort: function(port, scheme)
	{
		return false;
	},

	newURI: function(spec, charset, baseURI)
	{
		var uri = Components.classes["@mozilla.org/network/simple-uri;1"].
				createInstance(Components.interfaces.nsIURI);

		uri.spec = spec;
		return uri;
	},

	newChannel: function(input_uri)
	{
		var mpe = Components.classes["@fjsoft.at/MyPhoneExplorer;1"].
				getService(Components.interfaces.mpeIMyPhoneExplorer);

		var ios = Components.classes["@mozilla.org/network/io-service;1"].
				getService(Components.interfaces.nsIIOService);
		 
		var mpe_progdir = mpe.GetRegEntry("PathToExe");

		var i = mpe_progdir.lastIndexOf("\\");
		mpe_progdir = mpe_progdir.substring(0,i+1);	

		var mpe_file = input_uri.spec;
		mpe_file = mpe_file.slice(mpe_file.indexOf(":")+1);

		var file = Components.classes["@mozilla.org/file/local;1"].
				createInstance(Components.interfaces.nsILocalFile);

		file.initWithPath(mpe_progdir + mpe_file);

		var file_uri = ios.newFileURI(file);

		var channel = ios.newChannelFromURI(file_uri);

		if(mpe_file.match(/\.rdf$/)) channel.contentType = "text/xml";
		else if(mpe_file.match(/\.xpi$/)) channel.contentType = "application/x-xpinstall";

		return channel;
	},

	/* Components.interfaces.nsISupports */
  	QueryInterface : function handler_QI(aIID) 
  	{
    		if (aIID.equals(Components.interfaces.nsISupports) ||
        	    (Components.interfaces.nsIProtocolHandler && aIID.equals(Components.interfaces.nsIProtocolHandler))) {
      			return this;
    		}

    		throw Components.results.NS_ERROR_NO_INTERFACE;
  	}
};

var mpeProtocolFactory =
{
	createInstance : function(outer, iid)
	{
    		if (outer != null) {
      			throw Components.results.NS_ERROR_NO_AGGREGATION;
    		}

    		return new mpeProtocolHandler().QueryInterface(iid);
  	}
};

var mpeProtocolModule =
{
	registerSelf : function(compMgr, fileSpec, location, type)
  	{
    		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    		compMgr.registerFactoryLocation(MPE_PROTO_CLSID, MPE_PROTO_TITLE, MPE_PROTO_CONTRACTID, fileSpec, location, type);
  	},

  	unregisterSelf : function(compMgr, fileSpec, location)
  	{
    		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    		compMgr.unregisterFactoryLocation(MPE_PROTO_CLSID, fileSpec);
  	},

  	getClassObject : function(compMgr, cid, iid)
  	{
    		if (cid.equals(MPE_PROTO_CLSID)) {
      			return mpeProtocolFactory;
    		}

    		if (!iid.equals(Components.interfaces.nsIFactory)) {
      			throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    		}

    		throw Components.results.NS_ERROR_NO_INTERFACE;
  	},

  	canUnload : function(compMgr)
  	{
    		return true;
  	}
};


function NSGetModule(compMgr, fileSpec) {
  	return mpeProtocolModule;
}
